% for usage see crossvalidation script
function [IAll,mDistancesToPrototypesAll] = apply_model(oModel,mFullData)
if(length(oModel.options{1})>3)
    fKernel         = oModel.options{1}{4};
else
    'assume linear kernel'
    fKernel         = @(X,Y)X*Y';
end
iNrOfPrototypes = size(oModel.mAlpha,1);
%try    
    if(nargin>2)
        iSplits = 10;
    else
        iSplits = 1;
    end
    iNrOfAllDataPoints = size(mFullData,1);
    iStartBlock = 1;
    iBlockSize  = ceil(iNrOfAllDataPoints/iSplits);
    IAll        = zeros(iNrOfAllDataPoints,1); 
    mDistancesToPrototypesAll = zeros(iNrOfAllDataPoints, iNrOfPrototypes);

    %% check first which training points are in use in the alpha matrix
    vUsed    = [];
    for i=1:iNrOfPrototypes            
        IUsed    = find(oModel.mAlpha(i,:)>0);
        vUsed{i} = IUsed;
    end
    vTrainIndicesNeeded  = unique(sort(cell2mat(vUsed))); % according to alpha matrix
    
    mBoolIndicesNeededPerPrototype = false(iNrOfPrototypes,length(vTrainIndicesNeeded));
    for i=1:iNrOfPrototypes            
        mBoolIndicesNeededPerPrototype(i,:) = any(bsxfun(@eq,vTrainIndicesNeeded,vUsed{i}'),1);
    end    

    % some kernel information needed later on to calculate the distance to
    % new points
    KTrainNeededFull = fKernel(oModel.oTrain.data(vTrainIndicesNeeded,:),oModel.oTrain.data(vTrainIndicesNeeded,:));
        
    for(s=1:iSplits)
        if(iStartBlock+iBlockSize > iNrOfAllDataPoints)
            iBlockSize = iNrOfAllDataPoints - iStartBlock +1;
        end
        if(iBlockSize < 1)
            continue; % nothing more to do
        end
        dat = mFullData(iStartBlock:iStartBlock+iBlockSize-1,:);
        iNrOfDataPoints = size(dat,1);
        mDistancesToPrototypes = zeros(iNrOfDataPoints,iNrOfPrototypes);
        mAlpha   = oModel.mAlpha;
        
        % inner product of the prototypes k(w,w)
        vTemp       = zeros(iNrOfPrototypes,1);
        mAlphaShort = zeros(iNrOfPrototypes,length(vTrainIndicesNeeded));
        for i=1:iNrOfPrototypes            
                IUsed     = vUsed{i};
                vCurrentTrainIndicesUsed =  mBoolIndicesNeededPerPrototype(i,:);
                vTemp(i) = mAlpha(i,IUsed) * (KTrainNeededFull(vCurrentTrainIndicesUsed,vCurrentTrainIndicesUsed) * mAlpha(i,IUsed)');
                mAlphaShort(i,vCurrentTrainIndicesUsed) = mAlpha(i,vUsed{i});
        end
        
        % calculate distance between test points and prototypes
        iStepSize  = 10000;
        iStart     = 1;
        mTrainData = oModel.oTrain.data(vTrainIndicesNeeded,:);
        while(iStart<iNrOfDataPoints)
            iEnd       = min(iStart + iStepSize,iNrOfDataPoints);
            Ksmall     = fKernel(dat(iStart:iEnd,:),mTrainData);  % test to training data sub-kernel
            mDistancesToPrototypes([iStart:iEnd],:) = bsxfun(@plus,-2 * Ksmall * mAlphaShort',vTemp');
            iStart     = iEnd + 1;
        end    
        I = argmin(mDistancesToPrototypes,2);
        IAll(iStartBlock:iStartBlock+iBlockSize-1) = I; 
        mDistancesToPrototypesAll(iStartBlock:iStartBlock+iBlockSize-1,:) = mDistancesToPrototypes;
        iStartBlock = iStartBlock+iBlockSize;
    end
%catch sException
%    warning('ApplyCoreNG',sException.message)
%    warning('ApplyCoreNG','If you run out of memory - add - 1 -  as the last parameter as a few memory flag');
%end