function [DualCostFuncVal, meanDist]  = determine_cost_function_dual_approx( fKPtr, Assignment )

%
% Determines the value of the dual cost function for converged neurons
% It is also known as the intra-cluster distance
% In: 
%     fKPtr is a function pointer to a distance function
%   - Assignments 
%     vector with indices of the prototype the point is closest to
% Out:
%   - DualCostFuncVal - dual quantization error / intra-cluster distance 
%     (scales with the number of points
%
% A.Hasenfuss (c) 2008
% modified for CoreNG by F.-M. Schleif (c) 2012

Neurons = sort(unique(Assignment));

n = numel(Neurons);
meanDist = zeros(1,n);

for i = 1:n
  
  % Get receptive field
  nn = Neurons(i);
  idx = (Assignment == nn);
  
  % Number of elements in receptive field    
  vField = find(idx);
  num    = numel(vField);

  % shrink field;
  vFieldSamples = randsample(num,min(min(1000,ceil(0.1*num)),num));
  vField = vField(vFieldSamples);
  num    = numel(vField);
  vSum   = zeros(num-1,1);
  for(l=1:num-1)
      iRowIndex = vField(1);
      vField    = vField(2:end);
      vSum(l)   = sum(fKPtr( iRowIndex, vField));      
  end
  dSumDistances = 2*sum(vSum);
  
  % Mean of pairwise distances
  meanDist(i) = 0.5 * dSumDistances / num; 
  
end

DualCostFuncVal = 0.5 * sum( meanDist );
